--v1.0:  Convert checkboxes from actual widgets over to a table to use a slider with fixed checkboxes
gomlcheckboxes = {
		{name = "Serenity Mute", enabled = false, tooltip = "Mutes game audio while you have the Serenity buff on"},
		{name = "Arcanist Braedin", enabled = false, tooltip = "Hides Arcanist Braedin's /yells during the Wintergrasp battle"},
		{name = "Ability Spam", enabled = false, tooltip = "Hides messages about learning new spells/abilities"},
		{name = "Duel Messages", enabled = false, tooltip = "Hides duel chat messages"},
		{name = "Drinking Messages", enabled = false, tooltip = "Hides drinking chat messages"},
		{name = "Argent Tournament", enabled = false, tooltip = "Mutes Squire Cavin, The Black Knight, and Valiants/Champions"},
		{name = "Emblem/Mark Loot", enabled = false, tooltip = "Hides Badge/Emblem/Shard/Mark loot messages that don't pertain to you"},
		{name = "Popcap Games", enabled = false, tooltip = "Hides guildchat messages from Peggle and Bejeweled",},
		{name = "Necrosis", enabled = false, tooltip = "Filters out messages from the Necrosis addon"},
		{name = "Deserter Agitator", enabled = false, tooltip = "Hides speach from the Theramore Deserter Agitators"},
		{name = "Raid Joins/Leaves", enabled = false, tooltip = "Condenses multiple simultaneous raid join/leave messages into one message"},
		{name = "General Raid Mute", enabled = false, tooltip = "Hides General chat while in a raid group"},
		{name = "Fail Mods", enabled = false, tooltip = "Hides raid chat from \"Fail mods\""},
		{name = "Winter Time", enabled = false, tooltip = "Blocks guildchat messages from the WinterTime addon"},
		{name = "Auction House Messages", enabled = false, tooltip = "Blocks chat messages from the Auction House"},
		{name = "City Chatter", enabled = false, tooltip = "Blocks some annoying NPCs in cities, see CityChatter.lua in your GOML folder for a full list"},
		{name = "Trade Chat Filter", enabled = false, tooltip = "A simple attempt to keep only actual trade related messages in the Trade channel"},
		{name = "Achievement Clustering", enabled = false, tooltip = "Clusters multiple achievement awards into one message to reduce spam"},
		};

--A reference table to avoid looping through the checkboxes table to find a specific box
gomlreference = {
			["Serenity Mute"] = gomlcheckboxes[1];
			["Arcanist Braedin"] = gomlcheckboxes[2];
			["Ability Spam"] = gomlcheckboxes[3];
			["Duel Messages"] = gomlcheckboxes[4];
			["Drinking Messages"] = gomlcheckboxes[5];
			["Argent Tournament"] = gomlcheckboxes[6];
			["Emblem/Mark Loot"] = gomlcheckboxes[7];
			["Popcap Games"] = gomlcheckboxes[8];
			["Necrosis"] = gomlcheckboxes[9];
			["Deserter Agitator"] = gomlcheckboxes[10];
			["Raid Joins/Leaves"] = gomlcheckboxes[11];
			["General Raid Mute"] = gomlcheckboxes[12];
			["Fail Mods"] = gomlcheckboxes[13];
			["Winter Time"] = gomlcheckboxes[14];
			["Auction House Messages"] = gomlcheckboxes[15];
			["City Chatter"] = gomlcheckboxes[16];
			["Trade Chat Filter"] = gomlcheckboxes[17];
			["Achievement Clustering"] = gomlcheckboxes[18];
		};

--Frame setup function as pure LUA can't handle templates
function GOML_ConfigFrameSetup(frameobj)
	frameobj:SetScript("OnEnter", GOML_OnCheckBoxEnter);
	frameobj:SetScript("OnLeave", GOML_OnCheckBoxLeave);
	frameobj:SetScript("OnClick", GOML_OnCheckBoxClick);
end

--v1.0:  Slider frame update function
function GOMLConfig_SliderUpdate(self, delta)
	local current = GOMLConfigSlider:GetValue();

	if (delta < 0) and (current < math.ceil(#gomlcheckboxes / #GOMLConfig.buttons)) then
		GOMLConfigSlider:SetValue(current + 1);
	elseif (delta > 0) and (current > 1) then
		GOMLConfigSlider:SetValue(current - 1);
	end
end

function GOMLConfig_UpdateButtons(self, page)
	if not page then
		return;
	end

	local index = (page - 1) * #GOMLConfig.buttons;

	for i=1, #GOMLConfig.buttons do
		if gomlcheckboxes[index + i] then
			local btn = GOMLConfig.buttons[i];
			btn:Enable();
			getglobal(btn:GetName().."Text"):SetText(gomlcheckboxes[index + i].name);
			btn:SetChecked(gomlcheckboxes[index + i].enabled);
		else
			local btn = GOMLConfig.buttons[i];
			btn:Disable();
			getglobal(btn:GetName().."Text"):SetText(nil);
			btn:SetChecked(nil);	
		end
	end
end

--Create the config frame
GOMLConfig = { };
GOMLConfig.buttons = { };
local frame = CreateFrame("Frame", nil);
frame:SetScript("OnMouseWheel", GOMLConfig_SliderUpdate);
GOMLConfig.frame = frame;

--Create 10 checkbuttons
local chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton1", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.frame, "TOPLEFT", 15, -25);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[1] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton2", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[1], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[2] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton3", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[2], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[3] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton4", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[3], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[4] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton5", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[4], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[5] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton6", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[5], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[6] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton7", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[6], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[7] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton8", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[7], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[8] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton9", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[8], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[9] = chkBtn;

chkBtn = CreateFrame("CheckButton", "GOMLConfigCheckButton10", GOMLConfig.frame, "OptionsCheckButtonTemplate");
chkBtn:SetPoint("TOPLEFT", GOMLConfig.buttons[9], "BOTTOMLEFT", 0, -10);
GOML_ConfigFrameSetup(chkBtn);
GOMLConfig.buttons[10] = chkBtn;

--Create the Slider
local slider = CreateFrame("Slider", "GOMLConfigSlider", GOMLConfig.frame, "OptionsSliderTemplate");
slider:SetOrientation("VERTICAL");
slider:SetWidth(10);
slider:SetHeight(350);
slider:SetPoint("RIGHT", GOMLConfig.frame, "RIGHT", -25, 0); 
slider:SetMinMaxValues(1, math.ceil(#gomlcheckboxes / #GOMLConfig.buttons));
slider:SetValueStep(1.0);
slider:SetValue(1);
slider:SetScript("OnMouseWheel", GOMLConfig_SliderUpdate);
slider:SetScript("OnValueChanged", GOMLConfig_UpdateButtons);
GOMLConfigSliderLow:SetText("Looooow");
GOMLConfigSliderLow:SetPoint("BOTTOM", GOMLConfigSlider, "TOP");
GOMLConfigSliderHigh:SetText(tostring(math.ceil(#gomlcheckboxes / #GOMLConfig.buttons)));
GOMLConfigSliderText:SetText("1");


GOMLConfig_UpdateButtons(slider, 1)
GOMLConfig.frame:SetScript("OnShow", GOML_OnShow);

--Hook into default UI Interface Options
local panel = GOMLConfig.frame;
panel.name = "Get Off My Lawn!";
panel.okay = function (self) GOMLSaveConfig(); end;
InterfaceOptions_AddCategory(panel);