--Get Off My Lawn!  by Janus/Nirrudn

--Constants
GOML_UPDATE_INTERVAL = 3.0;

--Variables
local gomlsfxdefault;
local necrosisFilters;
local raidJoins = {};
local raidLeaves = {};
local L = GOMLLocalization;
local CL = GOMLChatterList;
local achievementsLocal = {};
local achievementsGuild = {};

--Create a core frame
CreateFrame("Frame", "GOMLCore");
GOMLCore:RegisterEvent("ADDON_LOADED");
GOMLCore:RegisterEvent("ZONE_CHANGED_NEW_AREA");
GOMLCore:RegisterEvent("WORLD_MAP_UPDATE");

--Event handlers
local GOMLhandlers = {
			["ADDON_LOADED"] = function(addon)
						if addon=="GetOffMyLawn" then
							GOMLSetup();

							--See if we can capture the current sound setting
							if GetCVar("Sound_EnableSFX") then
								gomlsfxdefault = 1;
							end
						end
					end,

			["UNIT_AURA"] = function()
						local aura = UnitBuff("player", L.SERENITY);
						GOML_OnAura(aura);
					end,

			["ZONE_CHANGED_NEW_AREA"] = function()
							if GetZoneText()==L.DRAGONBLIGHT and gomlconfig[1] then
								if not (GOMLCore:IsEventRegistered("UNIT_AURA")) then
									--See if we can capture the current sound setting
									if GetCVar("Sound_EnableSFX") then
										gomlsfxdefault = 1;
									end

									GOMLCore:RegisterEvent("UNIT_AURA");
								end
							else
								if GOMLCore:IsEventRegistered("UNIT_AURA") then
									GOMLCore:UnregisterEvent("UNIT_AURA");

									if gomlsfxdefault == 1 then
										SetCVar("Sound_EnableSFX", 1);
									end
								end
							end
						end,

			["WORLD_MAP_UPDATE"] = function()
							if GetZoneText()==L.DRAGONBLIGHT and gomlconfig[1] then
								if not (GOMLCore:IsEventRegistered("UNIT_AURA")) then
									--See if we can capture the current sound setting
									if GetCVar("Sound_EnableSFX") then
										gomlsfxdefault = 1;
									end

									GOMLCore:RegisterEvent("UNIT_AURA");
								end
							else
								if GOMLCore:IsEventRegistered("UNIT_AURA") then
									GOMLCore:UnregisterEvent("UNIT_AURA");

									if gomlsfxdefault == 1 then
										SetCVar("Sound_EnableSFX", 1);
									end
								end
							end
					end,
		};

function GOML_OnEvent(self, event, ...)
	local handler = GOMLhandlers[event];
	if handler then
		handler(...);
	end
end

GOMLCore:SetScript("OnEvent", GOML_OnEvent);

--Function to check if a passed name is on the chatter list
function GOMLIsChatterListed(name)
	if CL then
		for i, v in ipairs(CL) do
			if v == name then
				return true;
			end
		end
	end

	return false;
end


--Special function to check if the passed message matches any Necrosis speech
function GOMLNecrosisFilter(msg)

	if necrosisFilters then
		for i, pattern in ipairs(necrosisFilters) do
			if msg:match(pattern) then
				return true;
        		end
		end
	end

	return false;
end

function GOMLChatFilter_Achievement(frame, event, msg, name, ...)
	if gomlconfig[18] then
		if msg:match(L.ACHIEVEMENT_BROADCAST) then
			if event=="CHAT_MSG_ACHIEVEMENT" then
				achievementsLocal.window = frame;
				achievementsLocal.output = L.ACHIEVEMENT_MULTI_EARNED;
				achievementsLocal.capture = L.ACHIEVEMENT_BROADCAST;
				achievementsLocal.colorR = ChatTypeInfo["ACHIEVEMENT"].r;
				achievementsLocal.colorG = ChatTypeInfo["ACHIEVEMENT"].g;
				achievementsLocal.colorB = ChatTypeInfo["ACHIEVEMENT"].b;

				--Pull out the achievement link, use the ID as a key for a subTable
				local achLink = msg:match("\124cffffff00\124Hachievement.*");
				local achID = achLink:match(".*achievement:(%d+):.*");

				--Manually insert the name because links mess stuff up
				

				--DEBUG
				--DEFAULT_CHAT_FRAME:AddMessage("achLink is: " .. achLink);
				--DEFAULT_CHAT_FRAME:AddMessage("achID is: " .. achID);

				--Check if we have subTables, if not create them
				if achievementsLocal.subTables == nil then
					achievementsLocal.subTables = {};
				end

				if achievementsLocal.subTables[achID] == nil then
					achievementsLocal.subTables[achID] = {};
					achievementsLocal.subTables[achID].outputArgs = {};
				end

				local tempmsg = msg:gsub("^%%s", name, 1)

				--Manually insert the name because links mess stuff up
				tinsert(achievementsLocal.subTables[achID], tempmsg);

				--Store the achievement link as an outputArg on the subTable
				achievementsLocal.subTables[achID].outputArgs[1] = achLink;
	
				return true;

			elseif event=="CHAT_MSG_GUILD_ACHIEVEMENT" then
				achievementsGuild.window = frame;
				achievementsGuild.output = L.ACHIEVEMENT_MULTI_EARNED;
				achievementsGuild.capture = L.ACHIEVEMENT_BROADCAST;
				achievementsGuild.colorR = ChatTypeInfo["GUILD_ACHIEVEMENT"].r;
				achievementsGuild.colorG = ChatTypeInfo["GUILD_ACHIEVEMENT"].g;
				achievementsGuild.colorB = ChatTypeInfo["GUILD_ACHIEVEMENT"].b;

				--Pull out the achievement link, use the ID as a key for a subTable
				local achLink = msg:match("\124cffffff00\124Hachievement.*");
				local achID = achLink:match(".*achievement:(%d+):.*");

				--DEBUG
				--DEFAULT_CHAT_FRAME:AddMessage("achLink is: " .. achLink);
				--DEFAULT_CHAT_FRAME:AddMessage("achID is: " .. achID);
				--DEFAULT_CHAT_FRAME:AddMessage("name is:  " .. name);


				--Check if we have subTables, if not create them
				if achievementsGuild.subTables == nil then
					achievementsGuild.subTables = {};
				end

				if achievementsGuild.subTables[achID] == nil then
					achievementsGuild.subTables[achID] = {};
					achievementsGuild.subTables[achID].outputArgs = {};
				end

				local tempmsg = msg:gsub("^%%s", name, 1)

				tinsert(achievementsGuild.subTables[achID], tempmsg);

				--Store the achievement link as an outputArg on the subTable
				if achievementsGuild.subTables[achID].outputArgs then
					achievementsGuild.subTables[achID].outputArgs[1] = achLink;
				end
				return true;
			end
		end
	end

	return false;
end

function GOMLChatFilter_Guild(frame, event, msg, ...)
	if gomlconfig[8] and (strfind(msg, "^%[Peggle") or strfind(msg, "^%[Bejeweled")) then
		return true;
	end

	if gomlconfig[14] and msg:find("^%[WinterTime") then
		return true;
	end

	return false;
end

function GOMLChatFilter_Say(frame, event, msg, author, language, ...)
	if gomlconfig[9] and GOMLNecrosisFilter(msg) then
		return true;
	end

	return false;
end

function GOMLChatFilter_Yell(frame, event, msg, author, language, ...)
	if gomlconfig[9] and GOMLNecrosisFilter(msg) then
		return true;
	end

	return false;
end

function GOMLChatFilter_Raid(frame, event, msg, author, language, ...)
	if gomlconfig[13] and strfind(msg, ".* fail.* at .*") then
		return true;
	end

	if gomlconfig[9] and GOMLNecrosisFilter(msg) then
		return true;
	end

	return false;
end

function GOMLChatFilter_RaidLeader(frame, event, msg, author, language, ...)
	if gomlconfig[13] and strfind(msg, ".* fail.* at .*") then
		return true;
	end

	return false;
end

function GOMLChatFilter_Party(frame, event, msg, author, language, ...)
	if gomlconfig[9] and GOMLNecrosisFilter(msg) then
		return true;
	end

	return false;
end

function GOMLChatFilter_Channel(frame, event, msg, author, language, channel, ...)
	if gomlconfig[12] and UnitInRaid("player") and strfind(channel, GENERAL) and GetZoneText()~=L.WINTERGRASP then
			return true;
	end

	if gomlconfig[17] and channel:find(TRADE) then
		return GOMLParseTradeMessage(msg);
	end

	return false;
end

function GOMLChatFilter_System(frame, event, msg, ...)
	--System messages

	--Ability Spam box
	if gomlconfig[3] then
		if msg:match(L.ERR_LEARN_ABILITY_S) or
			msg:match(L.ERR_LEARN_SPELL_S) or
			msg:match(L.ERR_PET_LEARN_ABILITY_S) or
			msg:match(L.ERR_PET_LEARN_SPELL_S) or
			msg:match(L.ERR_SPELL_UNLEARNED_S) or
			msg:match(L.ERR_PET_SPELL_UNLEARNED_S) then

				return true;
		end
	end

	--Duel box
	if gomlconfig[4] then
		if msg:match(L.DUEL_WINNER_KNOCKOUT) or
			msg:match(L.DUEL_WINNER_RETREAT) then
				return true;
		end
	end

	--Drinking Messages box
	if gomlconfig[5] then
		if msg:match(L.DRUNK_MESSAGE_ITEM_OTHER1) or 
		   msg:match(L.DRUNK_MESSAGE_ITEM_OTHER2) or
		   msg:match(L.DRUNK_MESSAGE_ITEM_OTHER3) or
		   msg:match(L.DRUNK_MESSAGE_ITEM_OTHER4) or
		   msg:match(L.DRUNK_MESSAGE_OTHER1) or
		   msg:match(L.DRUNK_MESSAGE_OTHER2) or
		   msg:match(L.DRUNK_MESSAGE_OTHER3) or
		   msg:match(L.DRUNK_MESSAGE_OTHER4) then
			return true;
		end
	end

	--Raid join/leave stuff
	if gomlconfig[11] then
		if msg:match(L.ERR_RAID_MEMBER_ADDED_S) then
			--Store the window, output format, capture pattern, and colors in the table itself
			raidJoins.window = frame;
			raidJoins.output = L.RAID_MULTI_JOIN;
			raidJoins.capture = L.ERR_RAID_MEMBER_ADDED_S:gsub("%.%*", "(.*)");
			raidJoins.colorR = ChatTypeInfo["SYSTEM"].r;
			raidJoins.colorG = ChatTypeInfo["SYSTEM"].g;
			raidJoins.colorB = ChatTypeInfo["SYSTEM"].b;

			tinsert(raidJoins, msg);

			return true;
		end
		if msg:match(L.ERR_RAID_MEMBER_REMOVED_S) then
			--Store the window, output format, capture pattern, and colors in the table itself
			raidLeaves.window = frame;
			raidLeaves.output = L.RAID_MULTI_LEAVE;
			raidLeaves.capture = L.ERR_RAID_MEMBER_REMOVED_S:gsub("%.%*", "(.*)");
			raidLeaves.colorR = ChatTypeInfo["SYSTEM"].r;
			raidLeaves.colorG = ChatTypeInfo["SYSTEM"].g;
			raidLeaves.colorB = ChatTypeInfo["SYSTEM"].b;

			tinsert(raidLeaves, msg);

			return true;
		end
	end

	--AH Messages box
	if gomlconfig[15] and (GetNumPartyMembers() > 0 or UnitInRaid("player")) then
		if msg:match(L.ERR_AUCTION_SOLD_S) or
			msg:match(L.ERR_AUCTION_EXPIRED_S) or 
			msg:match(L.ERR_AUCTION_OUTBID_S) or
			msg:match(L.ERR_AUCTION_WON_S) then
				return true;
		end
	end

	return false;
end

function GOMLChatFilter_Loot(frame, event, msg, ...)
	--Debug garbage
	--DEFAULT_CHAT_FRAME:AddMessage(event .. msg);

	--Loot messages
	if gomlconfig[7] and (msg:match(L.LOOT_ITEM) or msg:match(L.CREATED_ITEM)) then
		--Loop through the item list and check if we match any items
		for k,v in ipairs(L.LOOTITEMLIST) do
			if msg:find(v) then
				return true;
			end
		end
	end

	return false;
end

function GOMLChatFilter_MonsterYell(frame, event, msg, author, ...)
	--NPC Yells
	if gomlconfig[2] and author==L.ARCANIST_BRAEDIN then
		return true;
	end

	if gomlconfig[6] and (author==L.SQUIRE_CAVIN or author==L.THE_BLACK_KNIGHT) then
		return true;
	end

	return false;
end

function GOMLChatFilter_MonsterSay(frame, event, msg, author, ...)
	--NPC Says
	if gomlconfig[6] and (strfind(author, L.VALIANT) or strfind(author, L.CHAMPION)) then
		return true;
	end

	if gomlconfig[10] and strfind(author, L.DESERTER_AGITATOR) then
		return true;
	end

	if gomlconfig[16] and GOMLIsChatterListed(author) then
		return true;
	end

	return false;
end

function GOMLMakeNecrosisFilters()
		--For the sake of efficient lookups these should be in their most commonly seen order
		necrosisFilters = {
					--Horse summons
					"I am summoning a steed from nightmare!",
					".*is giggling gloomily...",
					"Hey, I'm late! Let's find a horse that roxes!",
					"I call forth the flames of feet to make my travels swift!",

					--Pet summons
					"Fishing? Yes I love fishing... Look!",
					"I close my eyes, I move my fingers like that...",
					"And voila! Yes, yes, it is a fish, I can swear you!",
					".*is concentrating hard on Demoniac knowledge...",
					"I'll give you a soul if you come to me, Felguard! Please hear my command!",
					"Obey now, %a+!",
					".*looks in a bag, then throws a mysterious shard at %a+",
					"Please return in the Limbs you are from, Demon, and give me your power in exchange!",
					"%a+ ! %a+! Come on boy, come here! %a+!",
					"%a+ baby, please help me sweetheart!",
					"Oops, I'll probably need an idiot to be knocked for me...",
					"%a+, please help!",
					"%a+! HEEL! NOW!",
					"You crappy nasty little Imp, stop sulking and get over here to help! AND THAT'S AN ORDER!",

					--Summon messages
					"Welcome aboard, %a+, you are flying on the ~Succubus Air Lines~ to %a+...",
					"Air Hostesses and their lashes are at your service during your trip!",
					"Arcanum Taxi Cab! I am summoning %a+, please click on the portal.",
					"If you click on the portal, someone named %a+ will appear and do your job for you!",
					"If you do not want a sprawling, phlegm-looking, asthmatic creature to come from this portal, click on it to help %a+ find a path through Hell as quickly as possible!",
					"Arcane Taxi Cab for %a+, please click the portal for the slacker please.",

					--Soulstone messages
					"If you cherish the idea of a mass suicide, %a+ can now self-resurrect, so all should be fine. Go ahead.",
					"%a+ can go afk to drink a cup of coffee or something, soulstone is in place to allow for the wipe...",
					"Hmmm... %a+ is soulstoned... full of confidence tonight aren't we!!",
					"%a+ is Stoned... duuuude heavy!",
				};
end

--Set chat filters
function GOMLSetup()

	if not gomlconfig then
		gomlconfig = { };
	end

	ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL", GOMLChatFilter_MonsterYell);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", GOMLChatFilter_System)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_SAY", GOMLChatFilter_MonsterSay);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", GOMLChatFilter_Loot);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", GOMLChatFilter_Guild);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", GOMLChatFilter_Party);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", GOMLChatFilter_Raid);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", GOMLChatFilter_Channel);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", GOMLChatFilter_Yell);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", GOMLChatFilter_RaidLeader);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD_ACHIEVEMENT", GOMLChatFilter_Achievement);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_ACHIEVEMENT", GOMLChatFilter_Achievement);

	--Loop through the config saved variables and set up the check boxes' enabled values accordingly
	for i, v in ipairs(gomlcheckboxes) do
		gomlcheckboxes[i].enabled = gomlconfig[i] or false;
	end

	--Create the Necrosis filter table if it's enabled
	if gomlconfig[9] and not necrosisFilters then
		GOMLMakeNecrosisFilters();
	end

	--Set the OnUpdate script
	GOMLCore.TimeSinceLastUpdate = 0;
	GOMLCore:SetScript("OnUpdate", GOML_OnUpdate);
end

function GOMLSaveConfig()
	--Loop through the checkboxes table and put their enabled state into gomlconfig
	for i, v in ipairs(gomlcheckboxes) do
		gomlconfig[i] = gomlcheckboxes[i].enabled or false;
	end

	--Create the Necrosis filter table if it's enabled
	if gomlconfig[9] and not necrosisFilters then
		GOMLMakeNecrosisFilters();
	end
end

function GOML_OnShow()
	--Update the currently displayed checkboxes with their enabled status
	for i=1, #GOMLConfig.buttons do
		local btn = GOMLConfig.buttons[i];
		local btnText = getglobal(btn:GetName().."Text"):GetText();

		if btnText then
			btn:SetChecked(gomlreference[btnText].enabled);
		end
	end
end

function GOML_OnAura(aura)
	if aura then
		SetCVar("Sound_EnableSFX", 0);
	elseif gomlsfxdefault == 1 then
		SetCVar("Sound_EnableSFX", 1);
	end
end

function GOML_OnCheckBoxEnter(frameobj)
	--Pull off the button's number from the name
	local _, _, btnId = strfind(frameobj:GetName(), "GOMLConfigCheckButton(.*)");
	local index = (GOMLConfigSlider:GetValue() - 1) * #GOMLConfig.buttons;

	--Figure out the tooltip to display by the button number and slider value
	if gomlcheckboxes[index + tonumber(btnId)] then
		GameTooltip:SetOwner(frameobj, "ANCHOR_CURSOR");
		GameTooltip:SetText(gomlcheckboxes[index + tonumber(btnId)].tooltip);
		GameTooltip:Show();
	end

end

function GOML_OnCheckBoxClick(box)
	gomlreference[getglobal(box:GetName().."Text"):GetText()].enabled = box:GetChecked();
end

function GOML_OnCheckBoxLeave()
	GameTooltip:Hide();
end


--This function should return a string list of items pulled from a table using a specified pattern
function GOML_PullFromTable(messageTable, capturePattern)
	local pulls = ""; --Empty so we can concatenate

	--DEFAULT_CHAT_FRAME:AddMessage("capturePattern is " .. tostring(capturePattern));

	for i, message in ipairs(messageTable) do
		if message:match(capturePattern) then
			local _, _, foundName = message:find(capturePattern);

			if foundName then
				pulls = pulls .. foundName;

				--Pretty it up by inserting a comma and a space before the next item
				if i < #messageTable then
					pulls = pulls .. ", ";
				end
			end
		end
	end

	--If we found no names
	if pulls == "" then
		return nil;
	end

	return pulls;
end

function GOML_PrintWithTableArgs(printMe, outputArgs)
	if outputArgs then
		local result = printMe;

		for k,v in ipairs(outputArgs) do
				result = result:gsub("%%s$", v, 1);
		end

		return result;
	end

	return printMe;
end

function GOML_EvaluateTableForClustering(evalMe, window, output, capture, colorR, colorG, colorB)
	if evalMe == nil then
		DEFAULT_CHAT_FRAME:AddMessage("evalMe was nil");

		return;
	end

	if window then
		--Check the sub-tables first, and evaluate those if they exists
		if evalMe.subTables then
			--DEFAULT_CHAT_FRAME:AddMessage("subTables found...");
			for k,v in pairs(evalMe.subTables) do
				--DEFAULT_CHAT_FRAME:AddMessage("Executing evaluation on sub-table " .. tostring(k) .. " with value " .. tostring(v));
				GOML_EvaluateTableForClustering(v, window, output, capture, colorR, colorG, colorB);
			end
		else
			if #evalMe >= 2 then
				local namesList;
				local capturePattern = capture:gsub("%.%*", "(.*)", 1);
				local finalPrint;
	
				namesList = GOML_PullFromTable(evalMe, capturePattern);
	
				if namesList then
					--DEFAULT_CHAT_FRAME:AddMessage("namesList is " .. namesList);
					--DEFAULT_CHAT_FRAME:AddMessage("output is " .. output);
					if evalMe.outputArgs then
						finalPrint = GOML_PrintWithTableArgs(output, evalMe.outputArgs);
					else
						finalPrint = output;
					end

					--DEFAULT_CHAT_FRAME:AddMessage("finalPrint is " .. finalPrint);

					finalPrint = finalPrint:format(namesList);


					--DEFAULT_CHAT_FRAME:AddMessage("finalPrint is " .. finalPrint);

					window:AddMessage(finalPrint, colorR, colorG, colorB);
				end
			elseif #evalMe == 1 then
				--Just dump out the single message
				window:AddMessage(evalMe[1], colorR, colorG, colorB);
			end
		end

		wipe(evalMe);
	end
end

function GOML_OnUpdate(frame, time)
	if frame.TimeSinceLastUpdate then
		frame.TimeSinceLastUpdate = frame.TimeSinceLastUpdate + time;

		if frame.TimeSinceLastUpdate >= GOML_UPDATE_INTERVAL then
			GOML_EvaluateTableForClustering(raidJoins, raidJoins.window, raidJoins.output, raidJoins.capture, raidJoins.colorR, raidJoins.colorG, raidJoins.colorB);
			GOML_EvaluateTableForClustering(raidLeaves, raidLeaves.window, raidLeaves.output, raidLeaves.capture, raidLeaves.colorR, raidLeaves.colorG, raidLeaves.colorB);
			GOML_EvaluateTableForClustering(achievementsLocal, achievementsLocal.window, achievementsLocal.output, achievementsLocal.capture, achievementsLocal.colorR, achievementsLocal.colorG, achievementsLocal.colorB);
			GOML_EvaluateTableForClustering(achievementsGuild, achievementsGuild.window, achievementsGuild.output, achievementsGuild.capture, achievementsGuild.colorR, achievementsGuild.colorG, achievementsGuild.colorB);

			--[[wipe(raidJoins);
			wipe(raidLeaves);
			wipe(achievementsLocal);
			wipe(achievementsGuild);]]--

			frame.TimeSinceLastUpdate = 0;
		end
	end
end